package mschmidt.komo;
/*
 * Builds the actions and populates the menubar and coolbar when a new window is
 * opened. This work is factored into a separate class to avoid cluttering.
 * 
 * @since version 3.1
 */

import mschmidt.komo.preferences.PreferenceConstants;

import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;

public class ApplicationActionBarAdvisor extends ActionBarAdvisor {

	/**
	 * Initialize standard actions.
	 */
	private IWorkbenchAction aboutAction;

	private IWorkbenchAction exitAction;

	private IWorkbenchAction helpAction;

	private IWorkbenchAction introAction;

	private IWorkbenchAction preferencesAction;

	/**
	 * Initialize custom actions.
	 */
	private DbBackupAction dbBackupAction;
	
	private DbInfoAction dbInfoAction = null;

	private DbPasswordAction dbPasswordAction;
	
	private DbReindexAction dbReindexAction;
	
	private DbRestoreAction dbRestoreAction;
	
	private DbSetupAction dbSetupAction = null;

	private DbTestAction dbTestAction = null;

	private DbVacuumAction dbVacuumAction;

	/**
	 * Create instance of IActionBarConfigurer used for custom actions.
	 */
	private static IActionBarConfigurer thisConfigurer;

	/**
	 * @param configurer -
	 *            the ActionBarConfigurer.
	 */
	public ApplicationActionBarAdvisor(final IActionBarConfigurer configurer) {
		super(configurer);
		// Need to set a local instance of IActionBarConfigurer for custom
		// methods.
		thisConfigurer = configurer;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.application.
	 *      ActionBarAdvisor#makeActions(org.eclipse.ui.IWorkbenchWindow)
	 */
	@Override
	protected final void makeActions(final IWorkbenchWindow window) {

		aboutAction = ActionFactory.ABOUT.create(window);
		aboutAction.setImageDescriptor(KomoPlugin.getImageDescriptor(
				ApplicationImages.INFO_ICON));
		aboutAction.setToolTipText("Show program and system information");
		register(aboutAction);
		
		dbBackupAction = new DbBackupAction(window);
		register(dbBackupAction);

		dbInfoAction = new DbInfoAction(window);
		register(dbInfoAction);

		dbPasswordAction = new DbPasswordAction(window);
		register(dbPasswordAction);
		
		dbReindexAction = new DbReindexAction(window);
		register(dbReindexAction);
		
		dbRestoreAction = new DbRestoreAction(window);
		register(dbRestoreAction);

		dbSetupAction = new DbSetupAction(window);
		register(dbSetupAction);

		dbTestAction = new DbTestAction(window);
		register(dbTestAction);
		
		dbVacuumAction = new DbVacuumAction(window);
		register(dbVacuumAction);

		exitAction = ActionFactory.QUIT.create(window);
		exitAction.setToolTipText("Exit the program");
		register(exitAction);

		helpAction = ActionFactory.HELP_CONTENTS.create(window);
		helpAction.setToolTipText("Get online help");
		register(helpAction);

		introAction = ActionFactory.INTRO.create(window);
		introAction.setToolTipText("Show the Welcome screen");
		register(introAction);

		preferencesAction = ActionFactory.PREFERENCES.create(window);
		preferencesAction.setImageDescriptor(KomoPlugin.getImageDescriptor(
				ApplicationImages.CONFIGURE_ICON));
		preferencesAction.setToolTipText("Set program preferences");
		register(preferencesAction);
	}

	/*
	 * (non-Javadoc) Create the menubar.
	 * 
	 * @see org.eclipse.ui.application
	 *      .ActionBarAdvisor#fillMenuBar(org.eclipse.jface.action.IMenuManager)
	 */
	@Override
	protected final void fillMenuBar(final IMenuManager menuBar) {
		// Add File menu to menu bar

		// File menu
		MenuManager fileMenu = new MenuManager("&File",
				IWorkbenchActionConstants.M_FILE);
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.FILE_START));
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		fileMenu.add(preferencesAction);
		fileMenu.add(new Separator());
		fileMenu.add(exitAction);
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.FILE_END));
		menuBar.add(fileMenu);

		// DB Tools menu
		MenuManager dbToolsMenu = new MenuManager("&DB Tools",
				IWorkbenchActionConstants.MB_ADDITIONS);
		dbToolsMenu.add(dbVacuumAction);
		dbToolsMenu.add(dbReindexAction);
		dbToolsMenu.add(dbBackupAction);
		dbToolsMenu.add(dbRestoreAction);
		dbToolsMenu.add(new Separator());
		dbToolsMenu.add(dbTestAction);
		dbToolsMenu.add(dbSetupAction);
		menuBar.add(dbToolsMenu);

		// Help menu
		MenuManager helpMenu = new MenuManager("&Help",
				IWorkbenchActionConstants.M_HELP);
		helpMenu.add(new GroupMarker(IWorkbenchActionConstants.HELP_START));
		helpMenu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		helpMenu.add(aboutAction);
		helpMenu.add(helpAction);
		helpMenu.add(dbInfoAction);
		helpMenu.add(new Separator());
		helpMenu.add(introAction);
		helpMenu.add(new GroupMarker(IWorkbenchActionConstants.HELP_END));
		menuBar.add(helpMenu);
	}

	/**
	 * Create the coolbar.
	 * @see org.eclipse.ui.application.
	 *      ActionBarAdvisor#fillCoolBar(org.eclipse.jface.action.ICoolBarManager)
	 *      
	 * @param coolBar - the application coolbar
	 */
	@Override
	protected final void fillCoolBar(final ICoolBarManager coolBar) {
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_APP));
		// Application created group
		IToolBarManager appToolBar = new ToolBarManager(coolBar.getStyle());
		appToolBar.add(new Separator(IWorkbenchActionConstants.NEW_GROUP));
		appToolBar.add(dbTestAction);
		/*
		 * Add an Enter Password action to coolbar if user hasn't stored
		 * password in Preferences.
		 */
		if (!KomoPlugin.getDefault().getPreferenceStore().getBoolean(
				PreferenceConstants.DB_STORE_PW)) {
			appToolBar.add(dbPasswordAction);
		}
		coolBar.add(new ToolBarContributionItem(appToolBar));
	}

	/**
	 * Method to fill the system tray item popup menu.
	 * @param trayItem - the tray item menu manager
	 */
	protected final void fillTrayItem(IMenuManager trayItem) {
		trayItem.add(aboutAction);
		trayItem.add(exitAction);
	}
	/**
	 * This method writes messages to the status line.
	 * @param message - The message to be written.
	 * @param isError - True if this is an error message, false if not.
	 */
	public static final void postMessage(final String message, final boolean isError) {
		if (message == null || message.length() == 0) {
			return;
		}
		if (isError) {
			thisConfigurer.getStatusLineManager().setErrorMessage(
					KomoPlugin.getImageDescriptor(ApplicationImages.ERROR_ICON)
							.createImage(), message);
		} else {
			thisConfigurer.getStatusLineManager().setErrorMessage(null, "");
			thisConfigurer.getStatusLineManager().setMessage(
					KomoPlugin.getImageDescriptor(ApplicationImages.CHECK_ICON)
							.createImage(), message);
		}

	}

}

