package mschmidt.komo.preferences;

/*
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

import mschmidt.komo.Application;
import mschmidt.komo.ApplicationImages;
import mschmidt.komo.KomoPlugin;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

public class DbSettings extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage {

	/**
	 * Instantiate fields that will be accessed by performOK method.
	 */
	private BooleanFieldEditor storePWEditor;

	private StringFieldEditor userPasswordEditor;

	/**
	 * Constructor that creates this object.
	 */
	public DbSettings() {
		super(GRID);
		setDescription("Database Connection Settings");
		setTitle("Database Connection Settings");
		setImageDescriptor(KomoPlugin
				.getImageDescriptor(ApplicationImages.SERVER_ICON));
	}

	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public final void createFieldEditors() {

		final FileFieldEditor jdbcEditor = new FileFieldEditor(
				PreferenceConstants.JDBC_PATH, "JDBC File",
				getFieldEditorParent());
		jdbcEditor.setFileExtensions(new String[] { "*.jar" });
		final Label jdbcFileLabel = jdbcEditor
				.getLabelControl(getFieldEditorParent());
		jdbcFileLabel.setText("&JDBC File");
		jdbcFileLabel.setToolTipText("The PostgreSQL JDBC driver (jar file)");
		addField(jdbcEditor);

		final StringFieldEditor portEditor = new StringFieldEditor(
				PreferenceConstants.DB_PORT, "Server Port",
				getFieldEditorParent());
		portEditor.setErrorMessage("Server Port cannot be empty");
		portEditor.setEmptyStringAllowed(false);
		final Label serverPortLabel = portEditor
				.getLabelControl(getFieldEditorParent());
		serverPortLabel.setText("&Server Port");
		serverPortLabel.setToolTipText("The PostgreSQL server port");
		addField(portEditor);

		final StringFieldEditor serverAddressEditor = new StringFieldEditor(
				PreferenceConstants.DB_URL, "Server Address",
				getFieldEditorParent());
		serverAddressEditor.setErrorMessage("Server Address cannot be empty");
		serverAddressEditor.setEmptyStringAllowed(false);
		final Label serverAddressLabel = serverAddressEditor
				.getLabelControl(getFieldEditorParent());
		serverAddressLabel.setText("S&erver Address");
		serverAddressLabel.setToolTipText("The PostgreSQL server URL");
		addField(serverAddressEditor);

		final BooleanFieldEditor sslConnectionEditor = new BooleanFieldEditor(
				PreferenceConstants.DB_SSL, "SSL Connection",
				BooleanFieldEditor.SEPARATE_LABEL, getFieldEditorParent());
		final Label sslConnectionLabel = sslConnectionEditor
				.getLabelControl(getFieldEditorParent());
		sslConnectionLabel.setText("SS&L Connection");
		sslConnectionLabel
				.setToolTipText("Check to enable Secure Socket Layer connection");
		addField(sslConnectionEditor);

		final StringFieldEditor dbNameEditor = new StringFieldEditor(
				PreferenceConstants.DB_NAME, "Database Name:",
				getFieldEditorParent());
		dbNameEditor.setErrorMessage("Database name cannot be empty");
		dbNameEditor.setEmptyStringAllowed(false);
		final Label databaseNameLabel = dbNameEditor
				.getLabelControl(getFieldEditorParent());
		databaseNameLabel.setText("Da&tabase Name");
		databaseNameLabel.setToolTipText("The PostgreSQL database name");
		addField(dbNameEditor);

		final StringFieldEditor userNameEditor = new StringFieldEditor(
				PreferenceConstants.DB_USER, "User Name",
				getFieldEditorParent());
		userNameEditor.setErrorMessage("User Name cannot be empty");
		userNameEditor.setEmptyStringAllowed(false);
		final Label userNameLabel = userNameEditor
				.getLabelControl(getFieldEditorParent());
		userNameLabel.setText("&User Name");
		userNameLabel.setToolTipText("Your user name");
		addField(userNameEditor);

		userPasswordEditor = new StringFieldEditor(
				PreferenceConstants.DB_PASSWORD, "User Password",
				getFieldEditorParent());
		// prevent display of the password as it is entered for security reasons.
		final Text passwordText = userPasswordEditor
				.getTextControl(getFieldEditorParent());
		passwordText.setEchoChar('*');
		final Label userPasswordLabel = userPasswordEditor
				.getLabelControl(getFieldEditorParent());
		userPasswordLabel.setText("Use&r Password");
		userPasswordLabel.setToolTipText("Your user password");
		addField(userPasswordEditor);

		storePWEditor = new BooleanFieldEditor(PreferenceConstants.DB_STORE_PW,
				"Store password on this computer",
				BooleanFieldEditor.SEPARATE_LABEL, getFieldEditorParent());
		final Label storePWLabel = storePWEditor
				.getLabelControl(getFieldEditorParent());
		storePWLabel.setText("St&ore password locally");
		storePWLabel
				.setToolTipText("Check to store your password on this computer.\nWarning - the password may be accessed by others, \nwhich could be a security risk.");
		addField(storePWEditor);
	}

	/* (non-Javadoc)
	 * Makes sure that, if "Store Password locally" is true, that there is a 
	 * password to store.  Otherwise, OK action is not allowed.
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	@Override
	public final boolean performOk() {
		String newPW = userPasswordEditor.getStringValue();

		if (storePWEditor.getBooleanValue()) {
			if (newPW.length() == 0 || newPW == null) {
				this.setErrorMessage("Must enter a password to store.");

				// The OK button has been pressed and made inactive, 
				// so this workaround is needed to reset it to active.
				this.getContainer().updateButtons();
				newPW = "";
				return false;
			}
		} else {
			userPasswordEditor.setStringValue(
				getPreferenceStore().getDefaultString(
						PreferenceConstants.DB_PASSWORD));
		}
		Application.dbPassword.setPassword(newPW);
		return super.performOk();
	}

	/* 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(
	 * org.eclipse.ui.IWorkbench).
	 */
	public final void init(final IWorkbench workbench) {
		/*
		 * This line of code is needed here, rather than in the object 
		 * constructor, for preferences to work properly in the 
		 * RCP standalone application.
		 */
		setPreferenceStore(KomoPlugin.getDefault().getPreferenceStore());
		/*
		* Once user has accessed this page, turn off mandatory display of 
		* Intro screen on program startup.
		*/
		getPreferenceStore().putValue(
		PreferenceConstants.SHOW_INTRO, "false");
		
	}

}
